/*
 * monitor-calibration-tool, aka GammaaRRr
 * Copyright (C) 2003 Ross Burton <ross@burtonini.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#ifdef HAS_CONFIG_H
#include <config.h>
#endif

#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
 
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <glade/glade-xml.h>
#include "esco-disclosure.h"
#include "rb-gamma.h"
#include "rb-black-point-window.h"

static Display *display;
static int screen_number;

static GladeXML *xml;
static GtkWidget *dialog;
static GtkWidget *advanced;
static GtkWidget *gamma_scale, *red_scale, *green_scale, *blue_scale;
static GtkWidget *red_spin, *green_spin, *blue_spin;

static gboolean single = TRUE;
static double previous_gamma;

static void
init_x (GtkWindow *window)
{
  GdkScreen *gscreen;
  GdkDisplay *gdisplay;

  g_assert (window != NULL);

  gscreen = gtk_window_get_screen (window);
  g_assert (gscreen != NULL);
  screen_number = gdk_screen_get_number (gscreen);

  gdisplay = gdk_screen_get_display (gscreen);
  g_assert (gdisplay != NULL);
  display = gdk_x11_display_get_xdisplay (gdisplay);
}

/**
 * Get the screen's current gamma setting.
 */
static XF86VidModeGamma
get_gamma (void)
{
  static XF86VidModeGamma default_gamma = {1.0, 1.0, 1.0}; // TODO: urgh
  Bool result;
  XF86VidModeGamma gamma;
  result = XF86VidModeGetGamma (display, screen_number, &gamma);
  /* TODO: I'd like to pass back an anonymous struct here if possible */
  g_return_val_if_fail (result == True, default_gamma);
  return gamma;
}

/**
 * Set the current screen's gamma to the specified red, green and blue
 * values.
 */
static void
set_gamma (float r, float g, float b)
{
  Bool result;
  XF86VidModeGamma gamma;
  gamma.red = r;
  gamma.green = g;
  gamma.blue = b;
  result = XF86VidModeSetGamma (display, screen_number, &gamma);
  g_assert (result == True);
}

/**
 * Set the current screen's gamma to the specified red, green and blue
 * values.
 */
static void
set_gamma_offset (float r, float g, float b)
{
  Bool result;
  XF86VidModeGamma gamma;
  result = XF86VidModeGetGamma (display, screen_number, &gamma);
  g_assert (result == True);
  gamma.red += r;
  gamma.green += g;
  gamma.blue += b;
  result = XF86VidModeSetGamma (display, screen_number, &gamma);
  g_assert (result == True);
}

static void
update_ranges_from_gamma (void)
{
  XF86VidModeGamma gamma;
  double average;
  gamma = get_gamma ();
  average = (gamma.red + gamma.green + gamma.blue) / 3.0;
  gtk_range_set_value (GTK_RANGE (gamma_scale), average);
  gtk_range_set_value (GTK_RANGE (red_scale), gamma.red);
  gtk_range_set_value (GTK_RANGE (green_scale), gamma.green);
  gtk_range_set_value (GTK_RANGE (blue_scale), gamma.blue);
  previous_gamma = average;
}

/**
 * Callback when the overall gamma scale is changed
 */
void
on_gamma_value_changed (GtkWidget *scale)
{
  float gamma;
  g_return_if_fail (scale != NULL);
  gamma = gtk_range_get_value (GTK_RANGE (scale));
  if (single) {
    set_gamma (gamma, gamma, gamma);
  } else {
    double offset;
    offset = gamma - previous_gamma;
    set_gamma_offset (offset, offset, offset);
  }
  update_ranges_from_gamma ();
}

/**
 * Callback for when the individual channel gamma scales are changed.
 */
void
on_channel_gamma_value_changed (GtkWidget *scale)
{
  float r, g, b;
  r = gtk_range_get_value (GTK_RANGE (red_scale));
  g = gtk_range_get_value (GTK_RANGE (green_scale));
  b = gtk_range_get_value (GTK_RANGE (blue_scale));
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (red_spin), r);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (green_spin), g);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (blue_spin), b);
  set_gamma (r, g, b);
}

void
on_spin_changed (GtkSpinButton *spin, gpointer user_data)
{
  float r, g, b;
  r = gtk_spin_button_get_value (GTK_SPIN_BUTTON (red_spin));
  g = gtk_spin_button_get_value (GTK_SPIN_BUTTON (green_spin));
  b = gtk_spin_button_get_value (GTK_SPIN_BUTTON (blue_spin));
  gtk_range_set_value (GTK_RANGE (red_scale), r);
  gtk_range_set_value (GTK_RANGE (green_scale), g);
  gtk_range_set_value (GTK_RANGE (blue_scale), b);
  set_gamma (r, g, b);
}

void
disclosure_toggled (GtkToggleButton *disclosure, gpointer data)
{
  g_return_if_fail (disclosure != NULL);
  if (gtk_toggle_button_get_active (disclosure)) {
    float g;
    single = FALSE;
    //gtk_widget_set_sensitive (gamma_scale, FALSE);
    /* TODO: I get a redraw bug here -- is this due to ThinIce? */
    gtk_widget_show (advanced);
    /* Set the channel sliders */
    g = gtk_range_get_value (GTK_RANGE (gamma_scale));
    gtk_range_set_value (GTK_RANGE (red_scale), g);
    gtk_range_set_value (GTK_RANGE (green_scale), g);
    gtk_range_set_value (GTK_RANGE (blue_scale), g);
  } else { 
    float r, g, b;
    single = TRUE;
    //gtk_widget_set_sensitive (gamma_scale, TRUE);
    gtk_widget_hide (advanced);
    /* Set the slider to the average setting */
    r = gtk_range_get_value (GTK_RANGE (red_scale));
    g = gtk_range_get_value (GTK_RANGE (green_scale));
    b = gtk_range_get_value (GTK_RANGE (blue_scale));
    //g_signal_handlers_block_by_func (gamma_scale, &on_gamma_value_changed, NULL);
    gtk_range_set_value (GTK_RANGE (gamma_scale), (r+g+b)/3.0);
    //g_signal_handlers_unblock_by_func (gamma_scale, &on_gamma_value_changed, NULL);
  }
}

void
on_close_pressed (GtkWidget *button)
{
  gtk_main_quit ();
}

void
on_black_point_clicked (GtkButton *button, gpointer data)
{
  GtkWidget *window;
  window = rb_black_point_window_new ();
  gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (dialog));
  g_signal_connect (G_OBJECT (window), "key_press_event",
                    G_CALLBACK (gtk_widget_destroy), NULL);
  g_signal_connect (G_OBJECT (window), "button_press_event",
                    G_CALLBACK (gtk_widget_destroy), NULL);
  gtk_widget_show (window);
}

GtkWidget*
create_disclosure (void)
{
  return esco_disclosure_new_with_mnemonic ("_Advanced");
}

GtkWidget*
create_gamma_grey (void)
{
  GdkColor c = {0, 65535, 65535, 65535};
  return rb_gamma_new (c, 1.0);
}

GtkWidget*
create_gamma_red (void)
{
  GdkColor c = {0, 65535, 0, 0};
  return rb_gamma_new (c, 1.0);
}

GtkWidget*
create_gamma_green (void)
{
  GdkColor c = {0, 0, 65535, 0};
  return rb_gamma_new (c, 1.0);
}

GtkWidget*
create_gamma_blue (void)
{
  GdkColor c = {0, 0, 0, 65535};
  return rb_gamma_new (c, 1.0);
}

int
main (int argc, char **argv)
{
  float average;

  gtk_init(&argc, &argv);

  xml = glade_xml_new (GLADEDIR "/monitor-calibration.glade", NULL, NULL);
  g_assert (xml != NULL);
  glade_xml_signal_autoconnect (xml);

  dialog = glade_xml_get_widget (xml, "dialog");
  g_assert (dialog != NULL);
  advanced = glade_xml_get_widget (xml, "advanced");
  g_assert (advanced != NULL);

  gamma_scale = glade_xml_get_widget (xml, "gamma_scale");
  g_assert (gamma_scale != NULL);
  red_scale = glade_xml_get_widget (xml, "red_scale");
  g_assert (red_scale != NULL);
  green_scale = glade_xml_get_widget (xml, "green_scale");
  g_assert (green_scale != NULL);
  blue_scale = glade_xml_get_widget (xml, "blue_scale");
  g_assert (blue_scale != NULL);

  red_spin = glade_xml_get_widget (xml, "red_spin");
  g_assert (red_spin != NULL);
  green_spin = glade_xml_get_widget (xml, "green_spin");
  g_assert (green_spin != NULL);
  blue_spin = glade_xml_get_widget (xml, "blue_spin");
  g_assert (blue_spin != NULL);

  init_x (GTK_WINDOW(dialog));

  update_ranges_from_gamma ();

  gtk_widget_show_all (dialog);
  gtk_widget_hide (advanced);

  gtk_main ();
  return 0;
}
